/*************************************************
*                                                *
* SiTCP ROM                                      *
* Set values to the E2P ROM                      *
*                                                *
* 2008/03/25 Tomohisa Uchida                     *
*                                                *
*************************************************/
//#define DEBUG

#define DEFAULT_ADDRESS "192.168.0.16"
#define DEFAULT_UDP_PORT 0x1234
#define NUM_OF_RETRY 10

#define DATA_SIZE 64

#define BUF_SIZE 64

#include "bcp.h"
#include "bcp.c"
#include "myAtoi.c"

int bcp(char* ipAddr, unsigned int port, struct bcp_header*, char* sendData, char* rcvdBuf);

unsigned int myAtoi(char* str);

main(int argc, char* argv[]){

  /* Check inut values */
  if((argc < 4)){
    puts("\nSiTCP ROM: Writing program for the E2PROM");
    printf("Usage: %s <IP address> <UDP Port #>\n\n", argv[0]);
    }
  
  char* sitcpIpAddr;
  unsigned int udpPort;

  int i;
  
  /* Get IP address and port # of a SiTCP */

  if(argc<3){
    udpPort   = DEFAULT_UDP_PORT;
  }else{
    udpPort   = myAtoi(argv[3]);  
  }

  if(argc<2){
    sitcpIpAddr = DEFAULT_ADDRESS;
  }else{
    sitcpIpAddr = argv[2];
  }

  printf(" Set ...\n");
  printf("     \"%s\" for the IP address\n",sitcpIpAddr);
  printf("     \"%d\" for the RBCP port #\n\n",udpPort);

  /***********************************************************
   *                     Get values                          *
   ***********************************************************/
  FILE *fin;

  if((fin=fopen("setValues.txt","r"))==NULL){
    puts("Can't open setValues.txt");
    exit(EXIT_FAILURE);
  }

  int j=0, k;

  unsigned char sendBuf[BUF_SIZE];


  char temp[256];
  char value[10];

  while(j<DATA_SIZE){
    if(fgets(temp,sizeof(temp), fin)==NULL){
      puts("Too few chs. found");
      exit(EXIT_FAILURE);

    }else{
      i=0;

      if(temp[i]=='\0'){
	puts("Error #0: Detect a wrong line !");
	fclose(fin);
	exit(EXIT_FAILURE);
      }

      while(temp[i]==' ') i++;

      if((temp[i]!='/') && (temp[i]!='\n') && (temp[i]!='\r')){
	if((temp[i]<'0') && (temp[i]>'9')){
	  puts("Error #1:Detect a wrong expression !");
	  fclose(fin);
	  exit(EXIT_FAILURE);
	}

	for(k=0;k<10;k++){
	  value[k]=temp[i];
	  i++;
	}
	value[k]='\0';
	/*
	printf("[%d]=%d\n",j,myAtoi(value));
	*/
	sendBuf[j] =  myAtoi(value);
	j++;
      }
    }
  }

#ifdef DEBUG
  for(i=0; i<DATA_SIZE; i++){
    printf("[%x]=%d\n",i,sendBuf[i]);
  }
#endif

  fclose(fin);

  /***********************************************************
   *                        EEPROM WEN                       *
   ***********************************************************/
  unsigned char rcvdBuf[2048];
  struct bcp_header bcpHeader;

  bcpHeader.address=htonl(0xFFFFFCFFU);
  bcpHeader.command = 0x80;
  bcpHeader.length = 1;
  bcpHeader.id = 0;

  rcvdBuf[0] = 0;

  puts("\nEnable EEPROM to write data...");

  if(bcp(sitcpIpAddr,udpPort,&bcpHeader,rcvdBuf,rcvdBuf)<0){
    puts("\nbcp() failed");
    exit(EXIT_FAILURE);
  }

  if(rcvdBuf[3]!=bcpHeader.length){
    puts("\nbcp() failed: Access error");
    exit(EXIT_FAILURE);
  }

  /***********************************************************
   *                        Set data                         *
   ***********************************************************/
  bcpHeader.address=htonl(0xFFFFFC00U);
  bcpHeader.command = 0x80;
  bcpHeader.length = DATA_SIZE;
  bcpHeader.id = 1;

  puts("\nTransfer data to the EEPROM...");

  if(bcp(sitcpIpAddr,udpPort,&bcpHeader,sendBuf,rcvdBuf)<0){
    puts("\nbcp() failed");
    exit(EXIT_FAILURE);
  }

  if(rcvdBuf[3]!=bcpHeader.length){
    puts("\nbcp() failed: Access error");
    exit(EXIT_FAILURE);
  }

  /***********************************************************
   *                        Readback data                    *
   ***********************************************************/
  bcpHeader.address=htonl(0xFFFFFC00);
  bcpHeader.command = 0xC0;
  bcpHeader.length = DATA_SIZE;
  bcpHeader.id = 2;

  puts("\nRead back data from the EEPROM...");

  if(bcp(sitcpIpAddr,udpPort,&bcpHeader,rcvdBuf,rcvdBuf)<0){
    puts("\nbcp() failed");
    exit(EXIT_FAILURE);
  }

  if(rcvdBuf[3]!=bcpHeader.length){
    puts("\nbcp() failed: Access error");
    exit(EXIT_FAILURE);
  }

  puts("\nVerify data...");

  j=0;

	/* Verify data */
  for(i=0;i<DATA_SIZE;i++){
    if(rcvdBuf[i+sizeof(bcpHeader)]!=sendBuf[i]){
      j=1;
      printf("NG :Rx[%.2x]=%.2x, Tx[%.2x]=%.2x\n",i,rcvdBuf[i+sizeof(bcpHeader)], i,sendBuf[i]);
    }
  }

  if(j!=0){
    puts("\n  ****** Error ******");
  } else {
    puts("\n  ****** Done ! ******");
  }

  puts("\nFinished All Task");

}
